/* ====================================================================
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 2000-2003 The Apache Software Foundation.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://apache.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Apache" and "Apache Software Foundation", "Tapestry" 
 *    must not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache" 
 *    or "Tapestry", nor may "Apache" or "Tapestry" appear in their 
 *    name, without prior written permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE TAPESTRY CONTRIBUTOR COMMUNITY
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 *
 */

package org.apache.tapestry;

/**
 *  General wrapper for any exception (normal or runtime) that may occur during
 *  runtime processing for the application.  This is exception is used
 *  when the intent is to communicate a low-level failure to the user or
 *  developer; it is not expected to be caught.  The {@link #getRootCause() rootCause}
 *  property is a <em>nested</em> exception (Tapestry supported this concept
 *  long before the JDK did).
 *
 *  @author Howard Lewis Ship
 *  @version $Id: ApplicationRuntimeException.java,v 1.6 2003/08/06 14:28:48 hlship Exp $
 **/

public class ApplicationRuntimeException extends RuntimeException implements ILocatable
{
    private Throwable _rootCause;
    private transient ILocation _location;
    private transient Object _component;

    public ApplicationRuntimeException(Throwable rootCause)
    {
        this(rootCause.getMessage(), rootCause);
    }

    public ApplicationRuntimeException(String message)
    {
        this(message, null, null, null);
    }

    public ApplicationRuntimeException(String message, Throwable rootCause)
    {
        this(message, null, null, rootCause);
    }

    public ApplicationRuntimeException(
        String message,
        Object component,
        ILocation location,
        Throwable rootCause)
    {
        super(message);

        _rootCause = rootCause;
        _component = component;

        _location = Tapestry.findLocation(new Object[] { location, rootCause, component });
    }

    public ApplicationRuntimeException(String message, ILocation location, Throwable rootCause)
    {
        this(message, null, location, rootCause);
    }

    public Throwable getRootCause()
    {
        return _rootCause;
    }

    public ILocation getLocation()
    {
        return _location;
    }

    public Object getComponent()
    {
        return _component;
    }
}